@@ -149,6 +149,9 @@ class SysConfig(ConfigBase):
 
 class State(ConfigBase):
     def __init__(self):
+        if not os.path.exists('/var/db/hp/') and os.geteuid() == 0:
+            os.makedirs('/var/db/hp/')
+            os.system('chmod 644 /var/db/hp/')
         ConfigBase.__init__(self, '/var/db/hp/hplip.state')
 
 
