@@ -511,7 +511,7 @@ def run_plugin_command(required=True, plugin_reason=PLUGIN_REASON_NONE):
 
 
 def policykit_version():
-    if os.path.isdir("/usr/share/polkit-1"):
+    if os.path.isdir("/usr/local/share/polkit-1"):
         return 1
     elif os.path.isdir("/usr/share/PolicyKit"):
         return 0
