@@ -353,7 +353,7 @@ def get_xsane_version():
         output=p1.communicate()[0]
 
     if output:          
-        xsane_ver_pat =re.compile('''xsane-(.*) \(c\).*''')
+        xsane_ver_pat =re.compile('''xsane-(\d{1,}\.\d{1,}).*''')
         xsane_ver_info = output.splitlines()[0]
         if xsane_ver_pat.search(xsane_ver_info):
             installed_ver = xsane_ver_pat.search(xsane_ver_info).group(1)
@@ -373,9 +373,12 @@ def get_libpthread_version():
     except ImportError:
         return '-'
     else:
-        LIBC = ctypes.CDLL(ctypes.util.find_library('c'), use_errno=True)
-        LIBC.gnu_get_libc_version.restype = ctypes.c_char_p
-        return LIBC.gnu_get_libc_version()
+        if sys.platform.startswith('linux'):
+            LIBC = ctypes.CDLL(ctypes.util.find_library('c'), use_errno=True)
+            LIBC.gnu_get_libc_version.restype = ctypes.c_char_p
+            return LIBC.gnu_get_libc_version()
+        else:
+            return '-'
 
 def get_python_xml_version():
     try:
